<?php

namespace Drp\SimpleCms;

use Drp\SimpleCms\Contracts\IsManaged;
use Drp\SimpleCms\Contracts\Previewable;
use Drp\SimpleCms\Contracts\Publishable;
use Drp\SimpleCms\Contracts\ValidatesOnPublish;
use Illuminate\Database\Eloquent\Model;

abstract class Content extends Model implements Publishable, Previewable, IsManaged, ValidatesOnPublish
{
    use Publisher, Previewer;

    public function destroyAllVersion(): int
    {
        $versions = $this->where('content_id', $this->content_id)
            ->get()
            ->pluck('id')
            ->toArray();

        return $this->destroy($versions);
    }

    /**
     * @return mixed
     */
    public static function latestContent()
    {
        return self::all()
            ->groupBy('content_id')->map(function ($study) {
                return $study->sortBy('status')->first();
            })
            ->flatten();
    }

    public function visible(): bool
    {
        return $this->hasPublishedVersion();
    }

    // phpcs:disable SlevomatCodingStandard.TypeHints.TypeHintDeclaration.UselessDocComment

    /**
     * @SuppressWarnings(PHPMD.BooleanArgumentFlag)
     */
    public static function contentLabel(?bool $plural = false, ?bool $lowerCase = false): string
    {
        $label = str_replace('-', ' ', static::LABEL);

        if ($lowerCase) {
            $label = lcfirst($label);
        }

        return $plural ? str_plural($label) : str_singular($label);
    }

    // phpcs:enable SlevomatCodingStandard.TypeHints.TypeHintDeclaration.UselessDocComment

    public static function contentKey(): string
    {
        return static::KEY;
    }

    public static function relationshipName(): string
    {
        return lcfirst(studly_case(self::contentKey()));
    }

    public function getContentRouteParameters(): array
    {
        return [
            'type' => $this->contentKey(),
            'content_id' => $this->content_id,
        ];
    }

    public function getVersionRouteParameters(): array
    {
        return [
            'type' => $this->contentKey(),
            'content_id' => $this->id,
        ];
    }

    public function adminEditUrl(): string
    {
        return route($this->adminManageRouteName(), $this->getContentRouteParameters());
    }

    public function adminManageRouteName(): string
    {
        return config('simple_cms.admin_manage_route');
    }
}
