<?php

namespace Drp\SimpleCms;

use Drp\SimpleCms\Contracts\Publishable;
use Drp\SimpleCms\Exceptions\NoUpdatesMade;

class ContentManager implements \Drp\SimpleCms\Contracts\ContentManager
{
    /**
     * @var ContentFactory
     */
    protected $contentFactory;

    public function __construct(ContentFactory $contentFactory)
    {
        $this->contentFactory = $contentFactory;
    }

    /**
     * @param mixed $attributes
     */
    public function createContent(string $contentType, $attributes = null): Publishable
    {
        $content = $this->contentFactory->makeContentClass($contentType);
        $attributes['status'] = Publishable::STATUS_DRAFT;

        return $content->create($attributes);
    }

    public function updateContent(Publishable $content, ?array $attributes = null): Publishable
    {
        $draft = $this->getDraft($content, $attributes);
        $callback = $content->beforeSave();

        if ($callback instanceof \Closure) {
            call_user_func_array($callback, [$draft, $attributes]);
        }

        $draft->fill($attributes);
        $draft->save();

        $callback = $content->afterSave();

        if ($callback instanceof \Closure) {
            call_user_func_array($callback, [$draft, $attributes]);
        }

        return $draft;
    }

    /**
     * @param array|string|null $attributes
     */
    protected function getDraft(Publishable $content, $attributes = null): Publishable
    {
        $draft = $content->getDraftOrReviewVersion();

        if (empty($draft)) {
            //Check if the attributes are dirty against the published study
            if (!$content->isContentDirty($attributes)) {
                throw new NoUpdatesMade();
            }

            //If a draft version does not exist create one
            return $this->createDraftCopy($content);
        }

        //Check if the attributes are dirty against the draft study
        if (!$draft->isContentDirty($attributes)) {
            throw new NoUpdatesMade();
        }

        return $draft;
    }

    protected function createDraftCopy(Publishable $content): Publishable
    {
        $draft = $content->replicate();
        $draft->status = Publishable::STATUS_DRAFT;
        $draft->content_id = $content->content_id;
        $draft->save();

        return $draft;
    }
}
